@@ -3,7 +3,7 @@ module SciMLBaseForwardDiffExt
 using SciMLBase, ForwardDiff
 using ArrayInterface
 
-import SciMLBase: 
+import SciMLBase:
     wrapfun_oop, wrapfun_iip, isdualtype, value, DualEltypeChecker,
     AbstractTimeseriesSolution, NonlinearProblem, NonlinearLeastSquaresProblem,
     ODEProblem, SDEProblem, RODEProblem, DDEProblem, PDEProblem, DAEProblem,
@@ -14,31 +14,44 @@ eltypedual(x) = eltype(x) <: ForwardDiff.Dual
 isdualtype(::Type{<:ForwardDiff.Dual}) = true
 
 promote_dual(::Type{T}, ::Type{T2}) where {T <: ForwardDiff.Dual, T2} = T
-function promote_dual(::Type{T},
-        ::Type{T2}) where {T <: ForwardDiff.Dual, T2 <: ForwardDiff.Dual}
-    T
+function promote_dual(
+        ::Type{T},
+        ::Type{T2}
+    ) where {T <: ForwardDiff.Dual, T2 <: ForwardDiff.Dual}
+    return T
 end
 promote_dual(::Type{T}, ::Type{T2}) where {T, T2 <: ForwardDiff.Dual} = T2
 
-function promote_dual(::Type{T},
-        ::Type{T2}) where {T3, T4, V, V2 <: ForwardDiff.Dual, N, N2,
+function promote_dual(
+        ::Type{T},
+        ::Type{T2}
+    ) where {
+        T3, T4, V, V2 <: ForwardDiff.Dual, N, N2,
         T <: ForwardDiff.Dual{T3, V, N},
-        T2 <: ForwardDiff.Dual{T4, V2, N2}}
-    T2
+        T2 <: ForwardDiff.Dual{T4, V2, N2},
+    }
+    return T2
 end
-function promote_dual(::Type{T},
-        ::Type{T2}) where {T3, T4, V <: ForwardDiff.Dual, V2, N, N2,
+function promote_dual(
+        ::Type{T},
+        ::Type{T2}
+    ) where {
+        T3, T4, V <: ForwardDiff.Dual, V2, N, N2,
         T <: ForwardDiff.Dual{T3, V, N},
-        T2 <: ForwardDiff.Dual{T4, V2, N2}}
-    T
+        T2 <: ForwardDiff.Dual{T4, V2, N2},
+    }
+    return T
 end
-function promote_dual(::Type{T},
-        ::Type{T2}) where {
+function promote_dual(
+        ::Type{T},
+        ::Type{T2}
+    ) where {
         T3, V <: ForwardDiff.Dual, V2 <: ForwardDiff.Dual,
         N,
         T <: ForwardDiff.Dual{T3, V, N},
-        T2 <: ForwardDiff.Dual{T3, V2, N}}
-    ForwardDiff.Dual{T3, promote_dual(V, V2), N}
+        T2 <: ForwardDiff.Dual{T3, V2, N},
+    }
+    return ForwardDiff.Dual{T3, promote_dual(V, V2), N}
 end
 
 """
@@ -62,7 +75,7 @@ promote_dual(::Type{T}, ::Type{T2}) where {T, T2} = T
 # `mapreduce` hasn't received any such specialization.
 @inline diffeqmapreduce(f::F, op::OP, x::Tuple) where {F, OP} = reduce_tup(op, map(f, x))
 @inline function diffeqmapreduce(f::F, op::OP, x::NamedTuple) where {F, OP}
-    reduce_tup(op, map(f, x))
+    return reduce_tup(op, map(f, x))
 end
 # For other container types, we probably just want to call `mapreduce`
 @inline diffeqmapreduce(f::F, op::OP, x) where {F, OP} = mapreduce(f, op, x, init = Any)
@@ -76,7 +89,7 @@ const DUALCHECK_RECURSION_MAX = 10
 function (dec::DualEltypeChecker)(::Val{Y}) where {Y}
     isdefined(dec.x, Y) || return Any
     getval(dec.counter) >= DUALCHECK_RECURSION_MAX && return Any
-    anyeltypedual(getfield(dec.x, Y), Val{getval(dec.counter)})
+    return anyeltypedual(getfield(dec.x, Y), Val{getval(dec.counter)})
 end
 
 # Untyped dispatch: catch composite types, check all of their fields
@@ -103,18 +116,24 @@ themselves, for an example of how this can be confusing to a user see
 """
 @generated function anyeltypedual(x, ::Type{Val{counter}}) where {counter}
     x = x.name === Core.Compiler.typename(Type) ? x.parameters[1] : x
-    if isdualtype(x)
+    return if isdualtype(x)
         :($x)
     elseif fieldnames(x) === ()
         :(Any)
     elseif counter < DUALCHECK_RECURSION_MAX
-        T = diffeqmapreduce(x -> anyeltypedual(x, Val{counter + 1}), promote_dual,
-            x.parameters)
+        T = diffeqmapreduce(
+            x -> anyeltypedual(x, Val{counter + 1}), promote_dual,
+            x.parameters
+        )
         if T === Any || isconcretetype(T)
             :($T)
         else
-            :(diffeqmapreduce(DualEltypeChecker($x, $counter + 1), promote_dual,
-                map(Val, fieldnames($(typeof(x))))))
+            :(
+                diffeqmapreduce(
+                    DualEltypeChecker($x, $counter + 1), promote_dual,
+                    map(Val, fieldnames($(typeof(x))))
+                )
+            )
         end
     else
         :(Any)
@@ -122,104 +141,116 @@ themselves, for an example of how this can be confusing to a user see
 end
 
 const FORWARDDIFF_AUTODETECTION_FAILURE_MESSAGE = """
-                                Failed to automatically detect ForwardDiff compatability of
-                                the parameter object. In order for ForwardDiff.jl automatic
-                                differentiation to work on a solution object, the state of
-                                the differential equation or nonlinear solve (`u0`) needs to
-                                be converted to a Dual type which matches the values being
-                                differentiated. For example, for a loss function loss(p)
-                                where `p`` is a `Vector{Float64}`, this conversion is
-                                equivalent to:
+Failed to automatically detect ForwardDiff compatability of
+the parameter object. In order for ForwardDiff.jl automatic
+differentiation to work on a solution object, the state of
+the differential equation or nonlinear solve (`u0`) needs to
+be converted to a Dual type which matches the values being
+differentiated. For example, for a loss function loss(p)
+where `p`` is a `Vector{Float64}`, this conversion is
+equivalent to:
 
-                                ```julia
-                                # Convert u0 to match the new Dual element type of `p`
-                                _prob = remake(prob, u0 = eltype(p).(prob.u0))
-                                ```
+```julia
+# Convert u0 to match the new Dual element type of `p`
+_prob = remake(prob, u0 = eltype(p).(prob.u0))
+```
 
-                                In most cases, SciML tools are able to do this conversion
-                                automatically. However, it seems you have provided a
-                                parameter type for which this automatic conversion has failed.
+In most cases, SciML tools are able to do this conversion
+automatically. However, it seems you have provided a
+parameter type for which this automatic conversion has failed.
 
-                                To fix this, you can do the conversion yourself. For example,
-                                if you have a parameter vector being optimized `p` which is
-                                then put into an odd struct, you can manually convert `u0`
-                                to match `p`:
+To fix this, you can do the conversion yourself. For example,
+if you have a parameter vector being optimized `p` which is
+then put into an odd struct, you can manually convert `u0`
+to match `p`:
 
-                                ```julia
-                                function loss(p)
-                                    _prob = remake(prob, u0 = eltype(p).(prob.u0), p = MyStruct(p))
-                                    sol = solve(_prob, ...)
-                                    # do stuff on sol
-                                end
-                                ```
+```julia
+function loss(p)
+    _prob = remake(prob, u0 = eltype(p).(prob.u0), p = MyStruct(p))
+    sol = solve(_prob, ...)
+    # do stuff on sol
+end
+```
 
-                                Or you can define a dispatch on `SciMLBase.anyeltypedual`
-                                which tells the system what fields to interpret as the
-                                differentiable parts. For example, to support ODESolutions
-                                as parameters we tell it the data is `sol.u` and `sol.t` via:
+Or you can define a dispatch on `SciMLBase.anyeltypedual`
+which tells the system what fields to interpret as the
+differentiable parts. For example, to support ODESolutions
+as parameters we tell it the data is `sol.u` and `sol.t` via:
 
-                                ```julia
-                                function SciMLBase.anyeltypedual(sol::ODESolution, counter = 0)
-                                    SciMLBase.anyeltypedual((sol.u, sol.t))
-                                end
-                                ```
+```julia
+function SciMLBase.anyeltypedual(sol::ODESolution, counter = 0)
+    SciMLBase.anyeltypedual((sol.u, sol.t))
+end
+```
 
-                                To opt a type out of the dual checking, define an overload
-                                that returns Any. For example:
+To opt a type out of the dual checking, define an overload
+that returns Any. For example:
 
-                                ```julia
-                                function SciMLBase.anyeltypedual(::YourType, ::Type{Val{counter}}) where {counter}
-                                    Any
-                                end
-                                ```
+```julia
+function SciMLBase.anyeltypedual(::YourType, ::Type{Val{counter}}) where {counter}
+    Any
+end
+```
 
-                                If you have defined this on a common type which should
-                                be more generally supported, please open a pull request
-                                adding this dispatch. If you need help defining this dispatch,
-                                feel free to open an issue.
-                                """
+If you have defined this on a common type which should
+be more generally supported, please open a pull request
+adding this dispatch. If you need help defining this dispatch,
+feel free to open an issue.
+"""
 
 struct ForwardDiffAutomaticDetectionFailure <: Exception end
 
 function Base.showerror(io::IO, e::ForwardDiffAutomaticDetectionFailure)
-    print(io, FORWARDDIFF_AUTODETECTION_FAILURE_MESSAGE)
+    return print(io, FORWARDDIFF_AUTODETECTION_FAILURE_MESSAGE)
 end
 
 function anyeltypedual(::Type{Union{}})
     throw(ForwardDiffAutomaticDetectionFailure())
 end
 
-function anyeltypedual(::Type{<:AbstractTimeseriesSolution{T, N}},
-        ::Type{Val{counter}} = Val{0}) where {T, N, counter}
-    anyeltypedual(T)
+function anyeltypedual(
+        ::Type{<:AbstractTimeseriesSolution{T, N}},
+        ::Type{Val{counter}} = Val{0}
+    ) where {T, N, counter}
+    return anyeltypedual(T)
 end
 
 function anyeltypedual(
         ::Type{T},
-        ::Type{Val{counter}} = Val{0}) where {counter} where {T <:
-                                                              NonlinearProblem{
-        uType, iip, pType}} where {uType, iip, pType}
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter} where {
+        T <:
+        NonlinearProblem{
+            uType, iip, pType,
+        },
+    } where {uType, iip, pType}
     return anyeltypedual((uType, pType), Val{counter})
 end
 
 function anyeltypedual(
         ::Type{T},
-        ::Type{Val{counter}} = Val{0}) where {counter} where {T <:
-                                                              NonlinearLeastSquaresProblem{
-        uType, iip, pType}} where {uType, iip, pType}
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter} where {
+        T <:
+        NonlinearLeastSquaresProblem{
+            uType, iip, pType,
+        },
+    } where {uType, iip, pType}
     return anyeltypedual((uType, pType), Val{counter})
 end
 
-function anyeltypedual(x::SciMLBase.RecipesBase.AbstractPlot,
-        ::Type{Val{counter}} = Val{0}) where {counter}
-    Any
+function anyeltypedual(
+        x::SciMLBase.RecipesBase.AbstractPlot,
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter}
+    return Any
 end
 function anyeltypedual(x::Returns, ::Type{Val{counter}} = Val{0}) where {counter}
-    anyeltypedual(x.value, Val{counter})
+    return anyeltypedual(x.value, Val{counter})
 end
 
 Base.@assume_effects :foldable function __anyeltypedual(::Type{T}) where {T}
-    if T isa Union
+    return if T isa Union
         promote_dual(anyeltypedual(T.a), anyeltypedual(T.b))
     elseif hasproperty(T, :parameters)
         mapreduce(anyeltypedual, promote_dual, T.parameters; init = Any)
@@ -228,13 +259,17 @@ Base.@assume_effects :foldable function __anyeltypedual(::Type{T}) where {T}
     end
 end
 function anyeltypedual(::Type{T}, ::Type{Val{counter}} = Val{0}) where {counter} where {T}
-    __anyeltypedual(T)
+    return __anyeltypedual(T)
 end
 
-function anyeltypedual(::Type{T},
-        ::Type{Val{counter}} = Val{0}) where {counter} where {T <:
-                                                              Union{AbstractArray, Set}}
-    anyeltypedual(eltype(T))
+function anyeltypedual(
+        ::Type{T},
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter} where {
+        T <:
+        Union{AbstractArray, Set},
+    }
+    return anyeltypedual(eltype(T))
 end
 Base.@pure function __anyeltypedual_ntuple(::Type{T}) where {T <: NTuple}
     if isconcretetype(eltype(T))
@@ -247,68 +282,85 @@ Base.@pure function __anyeltypedual_ntuple(::Type{T}) where {T <: NTuple}
     end
 end
 function anyeltypedual(
-        ::Type{T}, ::Type{Val{counter}} = Val{0}) where {counter} where {T <: NTuple}
-    __anyeltypedual_ntuple(T)
+        ::Type{T}, ::Type{Val{counter}} = Val{0}
+    ) where {counter} where {T <: NTuple}
+    return __anyeltypedual_ntuple(T)
 end
 
 # Any in this context just means not Dual
 function anyeltypedual(
-        x::SciMLBase.NullParameters, ::Type{Val{counter}} = Val{0}) where {counter}
-    Any
+        x::SciMLBase.NullParameters, ::Type{Val{counter}} = Val{0}
+    ) where {counter}
+    return Any
 end
 
 function anyeltypedual(sol::RecursiveArrayTools.AbstractDiffEqArray, counter = 0)
-    diffeqmapreduce(anyeltypedual, promote_dual, (sol.u, sol.t))
+    return diffeqmapreduce(anyeltypedual, promote_dual, (sol.u, sol.t))
 end
 
-function anyeltypedual(prob::Union{ODEProblem, SDEProblem, RODEProblem, DDEProblem},
-        ::Type{Val{counter}} = Val{0}) where {counter}
-    anyeltypedual((prob.u0, prob.p, prob.tspan))
+function anyeltypedual(
+        prob::Union{ODEProblem, SDEProblem, RODEProblem, DDEProblem},
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter}
+    return anyeltypedual((prob.u0, prob.p, prob.tspan))
 end
 
 function anyeltypedual(
         prob::Union{NonlinearProblem, NonlinearLeastSquaresProblem, OptimizationProblem},
-        ::Type{Val{counter}} = Val{0}) where {counter}
-    anyeltypedual((prob.u0, prob.p))
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter}
+    return anyeltypedual((prob.u0, prob.p))
 end
 
 function anyeltypedual(x::Number, ::Type{Val{counter}} = Val{0}) where {counter}
-    anyeltypedual(typeof(x))
+    return anyeltypedual(typeof(x))
 end
 function anyeltypedual(
-        x::Union{String, Symbol}, ::Type{Val{counter}} = Val{0}) where {counter}
-    typeof(x)
+        x::Union{String, Symbol}, ::Type{Val{counter}} = Val{0}
+    ) where {counter}
+    return typeof(x)
 end
-function anyeltypedual(x::Union{AbstractArray{T}, Set{T}},
-        ::Type{Val{counter}} = Val{0}) where {counter} where {
+function anyeltypedual(
+        x::Union{AbstractArray{T}, Set{T}},
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter} where {
         T <:
-        Union{Number,
-        Symbol,
-        String}}
-    anyeltypedual(T)
-end
-function anyeltypedual(x::Union{AbstractArray{T}, Set{T}},
-        ::Type{Val{counter}} = Val{0}) where {counter} where {
-        T <: Union{
-        AbstractArray{
-            <:Number,
+        Union{
+            Number,
+            Symbol,
+            String,
         },
-        Set{
-            <:Number,
-        }}}
-    anyeltypedual(eltype(x))
+    }
+    return anyeltypedual(T)
 end
-function anyeltypedual(x::Union{AbstractArray{T}, Set{T}},
-        ::Type{Val{counter}} = Val{0}) where {counter} where {N, T <: NTuple{N, <:Number}}
-    anyeltypedual(eltype(x))
+function anyeltypedual(
+        x::Union{AbstractArray{T}, Set{T}},
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter} where {
+        T <: Union{
+            AbstractArray{
+                <:Number,
+            },
+            Set{
+                <:Number,
+            },
+        },
+    }
+    return anyeltypedual(eltype(x))
+end
+function anyeltypedual(
+        x::Union{AbstractArray{T}, Set{T}},
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter} where {N, T <: NTuple{N, <:Number}}
+    return anyeltypedual(eltype(x))
 end
 
 # Try to avoid this dispatch because it can lead to type inference issues when !isconcrete(eltype(x))
 function anyeltypedual(x::AbstractArray, ::Type{Val{counter}} = Val{0}) where {counter}
-    if isconcretetype(eltype(x))
+    return if isconcretetype(eltype(x))
         anyeltypedual(eltype(x))
     elseif !isempty(x) && all(i -> isassigned(x, i), 1:length(x)) &&
-           counter < DUALCHECK_RECURSION_MAX
+            counter < DUALCHECK_RECURSION_MAX
         _counter = Val{counter + 1}
         mapreduce(y -> anyeltypedual(y, _counter), promote_dual, x)
     else
@@ -319,7 +371,7 @@ function anyeltypedual(x::AbstractArray, ::Type{Val{counter}} = Val{0}) where {c
 end
 
 function anyeltypedual(x::Set, ::Type{Val{counter}} = Val{0}) where {counter}
-    if isconcretetype(eltype(x))
+    return if isconcretetype(eltype(x))
         anyeltypedual(eltype(x))
     else
         # This fallback to Any is required since otherwise we cannot handle `undef` in all cases
@@ -329,51 +381,66 @@ end
 
 function anyeltypedual(x::Tuple, ::Type{Val{counter}} = Val{0}) where {counter}
     # Handle the empty tuple case separately for inference and to avoid mapreduce error
-    if x === ()
+    return if x === ()
         Any
     else
         diffeqmapreduce(anyeltypedual, promote_dual, x)
     end
 end
 function anyeltypedual(x::AbstractDict, ::Type{Val{counter}} = Val{0}) where {counter}
-    isempty(x) ? eltype(values(x)) : mapreduce(anyeltypedual, promote_dual, values(x))
+    return isempty(x) ? eltype(values(x)) : mapreduce(anyeltypedual, promote_dual, values(x))
 end
 function anyeltypedual(x::NamedTuple, ::Type{Val{counter}} = Val{0}) where {counter}
-    anyeltypedual(values(x))
+    return anyeltypedual(values(x))
 end
 
 function anyeltypedual(
-        f::SciMLBase.AbstractSciMLFunction, ::Type{Val{counter}}) where {counter}
-    Any
+        f::SciMLBase.AbstractSciMLFunction, ::Type{Val{counter}}
+    ) where {counter}
+    return Any
 end
 
 anyeltypedual(::@Kwargs{}, ::Type{Val{counter}} = Val{0}) where {counter} = Any
 anyeltypedual(::Type{@Kwargs{}}, ::Type{Val{counter}} = Val{0}) where {counter} = Any
 
 # Opt out since these are using for preallocation, not differentiation
-function anyeltypedual(x::Union{ForwardDiff.AbstractConfig, Module},
-        ::Type{Val{counter}} = Val{0}) where {counter}
-    Any
+function anyeltypedual(
+        x::Union{ForwardDiff.AbstractConfig, Module},
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter}
+    return Any
 end
-function anyeltypedual(x::Type{T},
-        ::Type{Val{counter}} = Val{0}) where {counter} where {T <:
-                                                              ForwardDiff.AbstractConfig}
-    Any
+function anyeltypedual(
+        x::Type{T},
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter} where {
+        T <:
+        ForwardDiff.AbstractConfig,
+    }
+    return Any
 end
 
-function anyeltypedual(x::ForwardDiff.DiffResults.DiffResult,
-        ::Type{Val{counter}} = Val{0}) where {counter}
-    Any
+function anyeltypedual(
+        x::ForwardDiff.DiffResults.DiffResult,
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter}
+    return Any
 end
-function anyeltypedual(x::Type{T},
-        ::Type{Val{counter}} = Val{0}) where {counter} where {T <:
-                                                              ForwardDiff.DiffResults.DiffResult}
-    Any
+function anyeltypedual(
+        x::Type{T},
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter} where {
+        T <:
+        ForwardDiff.DiffResults.DiffResult,
+    }
+    return Any
 end
 
-function anyeltypedual(::Type{T},
-        ::Type{Val{counter}} = Val{0}) where {counter} where {T <: ForwardDiff.Dual}
-    T
+function anyeltypedual(
+        ::Type{T},
+        ::Type{Val{counter}} = Val{0}
+    ) where {counter} where {T <: ForwardDiff.Dual}
+    return T
 end
 
 value(x::Type{ForwardDiff.Dual{T, V, N}}) where {T, V, N} = V
@@ -386,7 +453,7 @@ sse(x::ForwardDiff.Dual) = sse(ForwardDiff.value(x)) + sum(sse, ForwardDiff.part
 
 function SciMLBase.totallength(x::ForwardDiff.Dual)
     return SciMLBase.totallength(ForwardDiff.value(x)) +
-           sum(SciMLBase.totallength, ForwardDiff.partials(x))
+        sum(SciMLBase.totallength, ForwardDiff.partials(x))
 end
 
 end
